package org.nissley.bluetooth_tester;

import android.Manifest;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;

import java.util.ArrayList;
import java.util.Date;
import java.util.Set;

public class ScrollingActivity extends AppCompatActivity implements GoogleApiClient.ConnectionCallbacks,
        GoogleApiClient.OnConnectionFailedListener,
        LocationListener {

    private BluetoothAdapter myBluetoothAdapter;
    private static final int REQUEST_ENABLE_BT = 1;
    /** Tag for Log */
    private static final String TAG = "DeviceListActivity";

    /**
     * Member object for the services
     */
    private BluetoothService btService = null;
    private ArrayList<GPSRow> GPSRows;
    private ArrayAdapter<GPSRow> btMessageAdapter;
    private ListView messageView;
    private LinearLayout loStatus;
    private TextView connectStatus;
    private BluetoothDevice btDevice = null;
    private Boolean isInternal = true;
    //private MyDBHandler dbHandler;
    private GoogleApiClient client;
    private LocationRequest locationRequest = new LocationRequest();
    private Location extGPSLocation;
    private Date previousDate = new Date();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scrolling);
        GPSRows = new ArrayList<>();
        final BluetoothManager btManager = (BluetoothManager)getSystemService(Context.BLUETOOTH_SERVICE);
        myBluetoothAdapter = btManager.getAdapter();
        if (myBluetoothAdapter == null) {
            Toast.makeText(getApplicationContext(),"Your device does not support Bluetooth",
                    Toast.LENGTH_LONG).show();
        } else {
            if (myBluetoothAdapter != null && !myBluetoothAdapter.isEnabled()) {
                Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
            }
        }
        btService = new BluetoothService(this, BTHandler);
        messageView = (ListView) findViewById(R.id.lvGPS1);
        btMessageAdapter = new GPSRowArrayAdapter(this, 0, GPSRows);
        messageView.setAdapter(btMessageAdapter);
        loStatus = (LinearLayout) findViewById(R.id.lo_ConnectStatus);
        connectStatus = (TextView) findViewById(R.id.tv_ConnectStatustxt);
        loStatus.setBackgroundColor(ContextCompat.getColor(this, R.color.disconnected));
        //dbHandler = new MyDBHandler(this, null, null, 1);
        // Create an instance of GoogleAPIClient.
        try {
            if (client == null) {
                client = new GoogleApiClient.Builder(this)
                        .addApi(LocationServices.API)
                        .addConnectionCallbacks(this)
                        .addOnConnectionFailedListener(this)
                        .build();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Button bt_update = (Button) findViewById(R.id.btn_pollSeconds);
        View.OnClickListener btn_OCL = new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                EditText etPollSeconds = (EditText) findViewById(R.id.et_pollSeconds);
                EditText etDelayedPollSeconds = (EditText) findViewById(R.id.et_delayedPollSec);
                String pollSeconds = etPollSeconds.getText().toString();
                String delayedPollSeconds = etDelayedPollSeconds.getText().toString();
                String sReturnMessage = "@13|" + pollSeconds + "|" + delayedPollSeconds + "\n";
                btService.write(sReturnMessage.getBytes());
                CheckBox cb_save = (CheckBox) findViewById(R.id.cb_save);
                if (cb_save.isChecked()) {
                    sReturnMessage = "@14|11\n";
                } else {
                    sReturnMessage = "@14|10\n";
                }
                btService.write(sReturnMessage.getBytes());
            }
        };
        bt_update.setOnClickListener(btn_OCL);
        Button bt_heartBeat = (Button) findViewById(R.id.btn_hb);
        View.OnClickListener btn_HB = new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String sReturnMessage = "@20|10\n";
                btService.write(sReturnMessage.getBytes());
            }
        };
        bt_heartBeat.setOnClickListener(btn_HB);
        if (myBluetoothAdapter.isEnabled()) {
            startBlueTooth();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //dbHandler.close();
        btService.stop();
        client.disconnect();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_scrolling, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        /*switch (item.getItemId()) {
            case R.id.mi_getDevice:
                btMessageAdapter.clear();
                //Intent btDevicesIntent = new Intent(this, BTDeviceDialog.class);
                //startActivityForResult(btDevicesIntent, 1);
                return true;
            case R.id.mi_delete_database:
                //dbHandler.clearGPSData();
                return true;
            case R.id.mi_databaseView:
                //Intent viewDatabase = new Intent(this, DatabaseActivity.class);
                //startActivity(viewDatabase);
                return true;
        }*/
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQUEST_ENABLE_BT) {
            if(resultCode == Activity.RESULT_OK){
                startBlueTooth();
            }
            if (resultCode == Activity.RESULT_CANCELED) {
                //Write your code if there's no result
            }
        }
    }

    private void startBlueTooth() {
        btDevice = myBluetoothAdapter.getRemoteDevice(findGPSbtMAC());
        if (btService.getState() != BluetoothService.STATE_NONE) {
            btService.stop();
        } else {
            btService.connect(btDevice);
        }
    }

    /**
     * The Handler that gets information back from the BluetoothChatService
     */
    private final Handler BTHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case Constants.MESSAGE_STATE_CHANGE:
                    switch (msg.arg1) {
                        case BluetoothService.STATE_CONNECTED:
                            Log.e(TAG, "State: Connected");
                            isInternal = true;
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.connected));
                            connectStatus.setText("Connected");
                            client.disconnect();
                            break;
                        case BluetoothService.STATE_CONNECTING:
                            Log.e(TAG, "State: Connecting");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.connecting));
                            connectStatus.setText("Connecting");
                            break;
                        case BluetoothService.STATE_NONE:
                            Log.e(TAG, "State: None");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.disconnected));
                            connectStatus.setText("None");
                            break;
                        case BluetoothService.STATE_LOST_CONNECTION:
                            Log.e(TAG, "State: Lost Connection");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.lost_connection));
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    btService.connect(btDevice);
                                }
                            }, 20000);
                            connectStatus.setText("Lost Connection");
                            client.connect();
                            break;
                        case BluetoothService.STATE_CONNECTION_FAILED:
                            Log.e(TAG, "State: Connection Failed");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.connection_failed));
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    btService.connect(btDevice);
                                }
                            }, 10000);
                            connectStatus.setText("Connection Failed");
                            break;
                    }
                    break;
                case Constants.MESSAGE_READ:
                    // construct a string from the valid bytes in the buffer
                    String readMessage = msg.obj.toString();
                    Log.e(TAG, "State: Message Read");
                    parseBTInput(readMessage.split("\\|"));
                    Date dateNow = new Date();
                    Long diff = dateNow.getTime() - previousDate.getTime();
                    Long diffSeconds = diff / 1000 % 60;
                    //GPSRow oRow = parseGPSData(readMessage);
                    //btMessageAdapter.add(oRow);
                    //String sReturnMsg = "12|" + readMessage.substring(3,18) + "\n";
                    Log.d(TAG, diffSeconds.toString());
                    //btService.write(sReturnMsg.getBytes());
                    break;
            }
        }
    };

    private void parseBTInput(String[] items) {
        Integer apiCode = intTryParse(items[0], 0);
        String sReturnMessage;
        switch (apiCode) {
            case 10:
            case 11:
                if (items.length > 8) {
                    //10 is live gps data and 11 is delayed gps data
                    btMessageAdapter.add(parseGPSData(items));
                    //12 is the gps data ack
                    sReturnMessage = "@12|" + items[1] + "|" + items[2] + "\n";
                    Log.d(TAG, sReturnMessage);
                    btService.write(sReturnMessage.getBytes());
                }
                break;
            case 20:
                //This is a heartbeat. Answer with 21|<input data>
                sReturnMessage = "@21|" + items[1] + "\n";
                Log.d(TAG, sReturnMessage);
                btService.write(sReturnMessage.getBytes());
                break;
            case 21:
                Log.d(TAG, "Heartbeat");

            case 90:
                //90 is an error message. No need to return anything
                btMessageAdapter.add(new GPSRow(apiCode, "", "", -1, -1, items[1], "", -1, -1, -1));
                break;
        }
    }

    private static Integer intTryParse(String text, Integer defaultValue) {
        try {
            return Integer.parseInt(text.trim());
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    private GPSRow parseGPSData(String[] items) {
        Integer status;
        Integer speed;
        Integer altitude;
        Integer accuracy;
        Integer sats;
        String date;
        String time;
        String lat;
        String lng;
        status = intTryParse(items[0], -1);
        extGPSLocation = new Location(String.valueOf(status));
        String dateTemp = items[1].trim();
        String month = dateTemp.substring(0, 2);
        String day = dateTemp.substring(2, 4);
        String year = dateTemp.substring(4);
        date = month + "/" + day + "/" + year;
        String timeTemp = items[2].trim();
        String hours = timeTemp.substring(0, 2);
        String minutes = timeTemp.substring(2, 4);
        String seconds = timeTemp.substring(4);
        time = hours + ":" + minutes + ":" + seconds + " GMT";
        lat = items[3].trim();
        lng = items[4].trim();
        speed = intTryParse(items[5], -1);
        altitude = intTryParse(items[6], -1);
        accuracy = intTryParse(items[7], -1);
        sats = intTryParse(items[8], -1);
        int entryType = 0;
        if (!isInternal) {
            entryType = 1;
        }
        GPSRow row = new GPSRow(status, date, time, entryType, sats, lat, lng, altitude, speed, accuracy);
        //dbHandler.addGPSData(row);
        return row;
    }

    private String findGPSbtMAC() {
        String macAddress = null;
        // Get a set of currently paired devices
        Set<BluetoothDevice> pairedDevices = myBluetoothAdapter.getBondedDevices();
        // If there are paired devices, add each one to the ArrayAdapter
        if (pairedDevices.size() > 0) {
            for (BluetoothDevice device : pairedDevices) {
                if (device.getName().toLowerCase().equals("tis-gps")) {
                    macAddress = device.getAddress();
                    break;
                }
            }
        }
        return macAddress;
    }

    @Override
    public void onLocationChanged(Location location) {
        final Location oLocation = location;
        String test = String.valueOf(oLocation.getLatitude());
        Log.d(TAG, "Lat = " + test + " " + oLocation.getTime());
    }

    //GoogleApiClient
    @Override
    public void onConnectionFailed(ConnectionResult bundle) {
        Log.d(TAG, "Google services connection failed");
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.d(TAG, "Google services connected");
        locationRequest.setInterval(10000);
        locationRequest.setFastestInterval(1000);
        locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            // Check Permissions Now
            final int REQUEST_LOCATION = 2;

            if (ActivityCompat.shouldShowRequestPermissionRationale(this,
                    Manifest.permission.ACCESS_FINE_LOCATION)) {
                // Display UI and wait for user interaction
            } else {
                ActivityCompat.requestPermissions(
                        this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                        REQUEST_LOCATION);
            }
            LocationServices.FusedLocationApi.requestLocationUpdates(
                    client, locationRequest, this);
        } else {
            LocationServices.FusedLocationApi.requestLocationUpdates(
                    client, locationRequest, this);
        }
    }

    @Override
    public void onConnectionSuspended(int i) {

    }
}